@@ -17,7 +17,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* $Id: pgm.h,v 1.22 2005/09/18 20:12:23 joerg_wunsch Exp $ */
+/* $Id: pgm.h,v 1.24 2005/11/03 22:37:37 joerg_wunsch Exp $ */
 
 #ifndef __pgm_h__
 #define __pgm_h__
@@ -81,6 +81,10 @@ typedef struct programmer_t {
   int  (*set_varef)      (struct programmer_t * pgm, double v);
   int  (*set_fosc)       (struct programmer_t * pgm, double v);
   int  (*set_sck_period) (struct programmer_t * pgm, double v);
+  int  (*setpin)         (struct programmer_t * pgm, int pin, int value);
+  int  (*getpin)         (struct programmer_t * pgm, int pin);
+  int  (*highpulsepin)   (struct programmer_t * pgm, int pin);
+  int  (*getexitspecs)   (struct programmer_t * pgm, char *s, int *set, int *clr);
   char config_file[PATH_MAX]; /* config file where defined */
   int  lineno;                /* config file line number */
   char flag;		      /* for private use of the programmer */
@@ -104,17 +108,7 @@ void usleep(unsigned long us);
 
 void gettimeofday(struct timeval*, void*z);
 
-#define rindex strrchr
-
 #endif /* __win32native_h */
 
 
-#if !defined(ppi_claim)
-#  define ppi_claim(pgm)
-#endif
-
-#if !defined(ppi_release)
-#  define ppi_release(pgm)
-#endif
-
 #endif
