@@ -1,6 +1,6 @@
 /*
  * avrdude - A Downloader/Uploader for AVR device programmers
- * Copyright (C) 2003  Theodore A. Roth  <troth@openavr.org>
+ * Copyright (C) 2003-2004  Theodore A. Roth  <troth@openavr.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* $Id: ser_posix.c,v 1.8 2004/06/24 11:05:07 kiwi64ajs Exp $ */
+/* $Id: ser_posix.c,v 1.13 2005/08/30 01:30:05 bdean Exp $ */
 
 /*
  * Posix serial interface for avrdude.
@@ -38,9 +38,13 @@
 #include <termios.h>
 #include <unistd.h>
 
+#include "serial.h"
+
 extern char *progname;
 extern int verbose;
 
+long serial_recv_timeout = 5000; /* ms */
+
 struct baud_mapping {
   long baud;
   speed_t speed;
@@ -76,7 +80,7 @@ static speed_t serial_baud_lookup(long baud)
   exit(1);
 }
 
-static int serial_setattr(int fd, long baud)
+static int ser_setspeed(int fd, long baud)
 {
   int rc;
   struct termios termios;
@@ -90,7 +94,7 @@ static int serial_setattr(int fd, long baud)
    */
   rc = tcgetattr(fd, &termios);
   if (rc < 0) {
-    fprintf(stderr, "%s: serial_setattr(): tcgetattr() failed, %s", 
+    fprintf(stderr, "%s: ser_setspeed(): tcgetattr() failed, %s", 
             progname, strerror(errno));
     return -errno;
   }
@@ -108,16 +112,24 @@ static int serial_setattr(int fd, long baud)
   
   rc = tcsetattr(fd, TCSANOW, &termios);
   if (rc < 0) {
-    fprintf(stderr, "%s: serial_setattr(): tcsetattr() failed, %s", 
+    fprintf(stderr, "%s: ser_setspeed(): tcsetattr() failed, %s", 
             progname, strerror(errno));
     return -errno;
   }
 
+#if 0
+  /*
+   * set non blocking mode
+   */
+  rc = fcntl(fd, F_GETFL, 0);
+  fcntl(fd, F_SETFL, rc | O_NONBLOCK);
+#endif
+
   return 0;
 }
 
 
-int serial_open(char * port, int baud)
+static int ser_open(char * port, long baud)
 {
   int rc;
   int fd;
@@ -127,7 +139,7 @@ int serial_open(char * port, int baud)
    */
   fd = open(port, O_RDWR | O_NOCTTY /*| O_NONBLOCK*/);
   if (fd < 0) {
-    fprintf(stderr, "%s: serial_open(): can't open device \"%s\": %s\n",
+    fprintf(stderr, "%s: ser_open(): can't open device \"%s\": %s\n",
             progname, port, strerror(errno));
     exit(1);
   }
@@ -135,10 +147,10 @@ int serial_open(char * port, int baud)
   /*
    * set serial line attributes
    */
-  rc = serial_setattr(fd, baud);
+  rc = ser_setspeed(fd, baud);
   if (rc) {
     fprintf(stderr, 
-            "%s: serial_open(): can't set attributes for device \"%s\"\n",
+            "%s: ser_open(): can't set attributes for device \"%s\"\n",
             progname, port);
     exit(1);
   }
@@ -147,7 +159,7 @@ int serial_open(char * port, int baud)
 }
 
 
-void serial_close(int fd)
+static void ser_close(int fd)
 {
   /* FIXME: Should really restore the terminal to original state here. */
 
@@ -155,14 +167,13 @@ void serial_close(int fd)
 }
 
 
-int serial_send(int fd, char * buf, size_t buflen)
+static int ser_send(int fd, unsigned char * buf, size_t buflen)
 {
-  struct timeval timeout;
+  struct timeval timeout, to2;
   fd_set wfds;
   int nfds;
   int rc;
-
-  char * p = buf;
+  unsigned char * p = buf;
   size_t len = buflen;
 
   if (!len)
@@ -191,88 +202,94 @@ int serial_send(int fd, char * buf, size_t buflen)
 
   timeout.tv_sec = 0;
   timeout.tv_usec = 500000;
+  to2 = timeout;
 
   while (len) {
+  reselect:
     FD_ZERO(&wfds);
     FD_SET(fd, &wfds);
 
-  reselect:
-    nfds = select(fd+1, NULL, &wfds, NULL, &timeout);
+    nfds = select(fd+1, NULL, &wfds, NULL, &to2);
     if (nfds == 0) {
-      fprintf(stderr,
-              "%s: serial_send(): programmer is not responding\n",
-              progname);
+      if (verbose >= 1)
+	fprintf(stderr,
+		"%s: ser_send(): programmer is not responding\n",
+		progname);
       exit(1);
     }
     else if (nfds == -1) {
-      if (errno == EINTR) {
+      if (errno == EINTR || errno == EAGAIN) {
         goto reselect;
       }
       else {
-        fprintf(stderr, "%s: serial_send(): select(): %s\n",
+        fprintf(stderr, "%s: ser_send(): select(): %s\n",
                 progname, strerror(errno));
         exit(1);
       }
     }
 
-    rc = write(fd, p, 1);
+    rc = write(fd, p, (len > 1024) ? 1024 : len);
     if (rc < 0) {
-      fprintf(stderr, "%s: serial_send(): write error: %s\n",
+      fprintf(stderr, "%s: ser_send(): write error: %s\n",
               progname, strerror(errno));
       exit(1);
     }
-    p++;
-    len--;
+    p += rc;
+    len -= rc;
   }
 
   return 0;
 }
 
 
-int serial_recv(int fd, char * buf, size_t buflen)
+static int ser_recv(int fd, unsigned char * buf, size_t buflen)
 {
-  struct timeval timeout;
+  struct timeval timeout, to2;
   fd_set rfds;
   int nfds;
   int rc;
-
-  char * p = buf;
+  unsigned char * p = buf;
   size_t len = 0;
 
-  timeout.tv_sec  = 5;
-  timeout.tv_usec = 0;
+  timeout.tv_sec  = serial_recv_timeout / 1000L;
+  timeout.tv_usec = (serial_recv_timeout % 1000L) * 1000;
+  to2 = timeout;
 
   while (len < buflen) {
+  reselect:
     FD_ZERO(&rfds);
     FD_SET(fd, &rfds);
 
-  reselect:
-    nfds = select(fd+1, &rfds, NULL, NULL, &timeout);
+    nfds = select(fd+1, &rfds, NULL, NULL, &to2);
     if (nfds == 0) {
-      fprintf(stderr, 
-              "%s: serial_recv(): programmer is not responding\n",
-              progname);
-      exit(1);
+      if (verbose > 1)
+	fprintf(stderr,
+		"%s: ser_recv(): programmer is not responding\n",
+		progname);
+      return -1;
     }
     else if (nfds == -1) {
-      if (errno == EINTR) {
+      if (errno == EINTR || errno == EAGAIN) {
+	fprintf(stderr,
+		"%s: ser_recv(): programmer is not responding,reselecting\n",
+		progname);
         goto reselect;
       }
       else {
-        fprintf(stderr, "%s: serial_recv(): select(): %s\n",
+        fprintf(stderr, "%s: ser_recv(): select(): %s\n",
                 progname, strerror(errno));
         exit(1);
       }
     }
 
-    rc = read(fd, p, 1);
+    rc = read(fd, p, (buflen - len > 1024) ? 1024 : buflen - len);
     if (rc < 0) {
-      fprintf(stderr, "%s: serial_recv(): read error: %s\n",
+      fprintf(stderr, "%s: ser_recv(): read error: %s\n",
               progname, strerror(errno));
       exit(1);
     }
-    p++;
-    len++;
+    p += rc;
+    len += rc;
   }
 
   p = buf;
@@ -301,7 +318,7 @@ int serial_recv(int fd, char * buf, size_t buflen)
 }
 
 
-int serial_drain(int fd, int display)
+static int ser_drain(int fd, int display)
 {
   struct timeval timeout;
   fd_set rfds;
@@ -334,7 +351,7 @@ int serial_drain(int fd, int display)
         goto reselect;
       }
       else {
-        fprintf(stderr, "%s: serial_drain(): select(): %s\n",
+        fprintf(stderr, "%s: ser_drain(): select(): %s\n",
                 progname, strerror(errno));
         exit(1);
       }
@@ -342,7 +359,7 @@ int serial_drain(int fd, int display)
 
     rc = read(fd, &buf, 1);
     if (rc < 0) {
-      fprintf(stderr, "%s: serial_drain(): read error: %s\n",
+      fprintf(stderr, "%s: ser_drain(): read error: %s\n",
               progname, strerror(errno));
       exit(1);
     }
@@ -354,4 +371,16 @@ int serial_drain(int fd, int display)
   return 0;
 }
 
+struct serial_device serial_serdev =
+{
+  .open = ser_open,
+  .setspeed = ser_setspeed,
+  .close = ser_close,
+  .send = ser_send,
+  .recv = ser_recv,
+  .drain = ser_drain,
+};
+
+struct serial_device *serdev = &serial_serdev;
+
 #endif  /* WIN32NATIVE */
